from translatepy import Translator
from translatepy.exceptions import TranslatepyException, UnknownLanguage

from .base import Translator as BaseTranslator


class TranslatePy(BaseTranslator):
    max_char = 1e10

    def __init__(self):
        self.translator = Translator()

    def translate(self, text, source_language, destination_language):
        try:
            result = self.translator.translate(
                text,
                source_language=source_language,
                destination_language=destination_language,
            )
            return result.result
        except UnknownLanguage as err:
            print("An error occured while searching for the language you passed in")
            print("Similarity:", round(err.similarity), "%")
            return
        except TranslatepyException:
            print("An error occured while translating with translatepy")
            return
        except Exception:
            print("An unknown error occured")
            return

    def quit(self):
        ...



import time
import logging

from typing import Optional
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.common.proxy import Proxy

from .base import Translator, TimeOutException
from .selenium_utils import (
    create_proxy,
    create_driver,
    TextArea,
    Button,
    Text,
)


class DeeplTranslator(Translator):
    url = "https://www.deepl.com/translator"
    max_char = 2800
    languages = dict(
        auto="Any language (detect)",
        bg="Bulgarian",
        zh="Chinese",
        cs="Czech",
        da="Danish",
        nl="Dutch",
        en="English",
        et="Estonian",
        fi="Finnish",
        fr="French",
        de="German",
        el="Greek",
        hu="Hungarian",
        it="Italian",
        ja="Japanese",
        lv="Latvian",
        lt="Lithuanian",
        pl="Polish",
        pt="Portuguese",
        ro="Romanian",
        ru="Russian",
        sk="Slovak",
        sl="Slovenian",
        es="Spanish",
        sv="Swedish",
    )

    def __init__(self, driver: Optional[WebDriver] = None):
        self.driver = driver
        if self.driver is None:
            proxy = create_proxy()
            self.driver = create_driver(proxy)

        logging.info(f"Going to {self.url}")
        self.driver.get(self.url)

        self.input_lang_from = TextArea(
            self.driver, "CLASS_NAME", "lmt__source_textarea"
        )
        self.input_destination_language = TextArea(
            self.driver, "CLASS_NAME", "lmt__target_textarea"
        )

    def _set_source_language(self, language: str) -> None:
        self._set_language(language, "lmt__language_select--source")

    def _set_destination_language(self, language: str) -> None:
        self._set_language(language, "lmt__language_select--target")

    def _set_language(self, language: str, dropdown_class: str) -> None:
        # Click the languages dropdown button
        Button(self.driver, "CLASS_NAME", dropdown_class).click()

        # Get the language button to click based on is dl-test property or the text in the button
        xpath_by_property = f"//button[@dl-test='translator-lang-option-{language}']"
        x_path_by_text = f"//button[text()='{self.languages[language]}']"
        xpath = f"{xpath_by_property} | {x_path_by_text}"

        # Click the wanted language button
        Button(self.driver, "XPATH", xpath).click()

    def _is_translated(self, original: str, translation: str) -> bool:
        return (
            len(translation) != 0
            and "[...]" not in translation
            and len(original.splitlines()) == len(translation.splitlines())
            and original != translation
        )

    def translate(self, text: str, source_language: str, destination_language: str):
        self._set_source_language(source_language)
        self._set_destination_language(destination_language)

        clean_text = text.replace("[...]", "@[.]@")

        self.input_lang_from.write((clean_text))

        # Maximun number of iterations 60 seconds
        for _ in range(60):
            translation = self.input_destination_language.value

            if self._is_translated(clean_text, translation):
                return translation.replace("@[.]@", "[...]")
            time.sleep(1)

        self.quit()
        raise TimeOutException("Translation timed out")

    def quit(self):
        self.driver.quit()



from setuptools import setup, find_packages

with open("requirements.txt") as f:
    requirements = f.read().splitlines()

setup(
    name="srtranslator",
    description="Traslate a .SRT file using any custom translator",
    url="https://github.com/sinedie/SRTranslator",
    version="0.1.0",
    author="EAR",
    author_email="sinedie@protonmail.com",
    license="FREE",
    python_requires=">=3.6",
    install_requires=requirements,
    packages=find_packages(),
    keywords=["python", "srt", "languages", "translator", "subtitles"],
)



import sys
import logging

from typing import Optional
from fp.fp import FreeProxy
from selenium import webdriver
from webdriverdownloader import GeckoDriverDownloader
from selenium.webdriver.remote.webdriver import WebDriver
from selenium.webdriver.support.ui import WebDriverWait
from selenium.webdriver.common.by import By
from selenium.webdriver.common.proxy import Proxy, ProxyType
from selenium.common.exceptions import WebDriverException
from selenium.common.exceptions import TimeoutException


def create_proxy() -> Proxy:
    """Creates a new proxy to use with a selenium driver and avoid get banned

    Returns:
        Proxy: Selenium WebDriver proxy
    """
    logging.info("Getting a new Proxy from https://www.sslproxies.org/")
    proxy = FreeProxy().get()
    proxy = Proxy(
        dict(
            proxyType=ProxyType.MANUAL,
            httpProxy=proxy,
            ftpProxy=proxy,
            sslProxy=proxy,
            noProxy="",
        )
    )
    return proxy


def create_driver(proxy: Optional[Proxy] = None) -> WebDriver:
    """Creates a new Firefox selenium webdriver. Install geckodriver if not in path

    Args:
        proxy (Optional[Proxy], optional): Selenium WebDriver proxy. Defaults to None.

    Returns:
        WebDriver: Selenium WebDriver
    """
    logging.info("Creating Selenium Webdriver instance")
    try:
        driver = webdriver.Firefox(proxy=proxy)
    except WebDriverException:
        logging.info("Installing Firefox GeckoDriver cause it isn't installed")
        gdd = GeckoDriverDownloader()
        gdd.download_and_install()

        driver = webdriver.Firefox(proxy=proxy)

    driver.maximize_window()
    return driver


class BaseElement:
    def __init__(
        self,
        driver: webdriver,
        locate_by: str,
        locate_value: str,
        multiple: bool = False,
        wait_time: int = 100,
    ) -> None:

        self.driver = driver
        locator = (getattr(By, locate_by.upper(), "id"), locate_value)
        find_element = driver.find_elements if multiple else driver.find_element
        try:
            WebDriverWait(driver, wait_time).until(
                lambda driver: find_element(*locator)
            )
            self.element = find_element(*locator)
        except TimeoutException:
            print(f"Timed out trying to get element ({locate_by} = {locate_value})")
            logging.info("Closing browser")
            driver.quit()
            sys.exit()


class Text(BaseElement):
    @property
    def text(self) -> str:
        return self.element.get_attribute("text")


class TextArea(BaseElement):
    def write(self, value: str) -> None:
        self.element.clear()
        self.element.send_keys(*value)

    @property
    def value(self) -> None:
        return self.element.get_attribute("value")


class Button(BaseElement):
    def click(self) -> None:
        try:
            can_click = getattr(self.element, "click", None)
            if callable(can_click):
                self.element.click()
        except:
            # Using javascript if usual click function does not work
            self.driver.execute_script("arguments[0].click();", self.element)
